@@ -500,10 +500,7 @@ set_attributes_from_tagsets (GFileInfo *info)
 	clear_studip_comments_from_tagset (info, _DESCRIPTION_TAG_NAMES);
 	clear_studip_comments_from_tagset (info, _TITLE_TAG_NAMES);
 
-	set_attribute_from_tagset (info, "general::datetime", _LAST_DATE_TAG_NAMES);
-	if (g_file_info_get_attribute_object (info, "general::datetime") == NULL)
-		set_attribute_from_tagset (info, "general::datetime", _ORIGINAL_DATE_TAG_NAMES);
-
+	set_attribute_from_tagset (info, "general::datetime", _ORIGINAL_DATE_TAG_NAMES);
 	set_attribute_from_tagset (info, "general::description", _DESCRIPTION_TAG_NAMES);
 	set_attribute_from_tagset (info, "general::title", _TITLE_TAG_NAMES);
 
@@ -663,9 +660,10 @@ exiv2_read_metadata (Exiv2::Image::AutoPtr  image,
  */
 extern "C"
 gboolean
-exiv2_read_metadata_from_file (GFile      *file,
-			       GFileInfo  *info,
-			       GError    **error)
+exiv2_read_metadata_from_file (GFile         *file,
+			       GFileInfo     *info,
+			       GCancellable  *cancellable,
+			       GError       **error)
 {
 	try {
 		char *path;
@@ -726,6 +724,28 @@ exiv2_read_metadata_from_buffer (void       *buffer,
 }
 
 
+extern "C"
+GFile *
+exiv2_get_sidecar (GFile *file)
+{
+	char  *uri;
+	char  *uri_wo_ext;
+	char  *sidecar_uri;
+	GFile *sidecar;
+
+	uri = g_file_get_uri (file);
+	uri_wo_ext = _g_uri_remove_extension (uri);
+	sidecar_uri = g_strconcat (uri_wo_ext, ".xmp", NULL);
+	sidecar = g_file_new_for_uri (sidecar_uri);
+
+	g_free (sidecar_uri);
+	g_free (uri_wo_ext);
+	g_free (uri);
+
+	return sidecar;
+}
+
+
 extern "C"
 gboolean
 exiv2_read_sidecar (GFile     *file,
@@ -815,12 +835,12 @@ const char *
 gth_main_get_metadata_type (gpointer    metadata,
 			    const char *attribute)
 {
-	const char      *value_type;
+	const char      *value_type = NULL;
 	GthMetadataInfo *metadatum_info;
 
 	if (GTH_IS_METADATA (metadata)) {
 		value_type = gth_metadata_get_value_type (GTH_METADATA (metadata));
-		if (g_strcmp0 (value_type, "Undefined") == 0)
+		if ((g_strcmp0 (value_type, "Undefined") == 0) || (g_strcmp0 (value_type, "") == 0))
 			value_type = NULL;
 
 		if (value_type != NULL)
@@ -835,13 +855,48 @@ gth_main_get_metadata_type (gpointer    metadata,
 }
 
 
+static void
+dump_exif_data (Exiv2::ExifData &exifData,
+		const char      *prefix)
+{
+	std::cout << prefix << "\n";
+
+	try {
+		if (exifData.empty()) {
+			throw Exiv2::Error(1, " No Exif data found in the file");
+		}
+		Exiv2::ExifData::const_iterator end = exifData.end();
+		for (Exiv2::ExifData::const_iterator i = exifData.begin(); i != end; ++i) {
+			const char* tn = i->typeName();
+			std::cout << std::setw(44) << std::setfill(' ') << std::left
+				  << i->key() << " "
+				  << "0x" << std::setw(4) << std::setfill('0') << std::right
+				  << std::hex << i->tag() << " "
+				  << std::setw(9) << std::setfill(' ') << std::left
+				  << (tn ? tn : "Unknown") << " "
+				  << std::dec << std::setw(3)
+				  << std::setfill(' ') << std::right
+				  << i->count() << "  "
+				  << std::dec << i->value()
+				  << "\n";
+		}
+		std::cout << "\n";
+	}
+	catch (Exiv2::Error& e) {
+	    std::cout << "Caught Exiv2 exception '" << e.what() << "'\n";
+	    return;
+	}
+}
+
+
 static Exiv2::DataBuf
 exiv2_write_metadata_private (Exiv2::Image::AutoPtr  image,
 			      GFileInfo             *info,
 			      GdkPixbuf             *pixbuf)
 {
-	char **attributes;
-	int    i;
+	static char  *software_name = NULL;
+	char        **attributes;
+	int           i;
 
 	image->clearMetadata();
 
@@ -900,7 +955,9 @@ exiv2_write_metadata_private (Exiv2::Image::AutoPtr  image,
 
 	// Overwrite the software tag
 
-	ed["Exif.Image.Software"] = PACKAGE " " VERSION;
+	if (software_name == NULL)
+		software_name = g_strconcat (g_get_application_name (), " ", VERSION, NULL);
+	ed["Exif.Image.Software"] = software_name;
 
 	// Update the dimension tags with actual image values
 
