@@ -34,7 +34,6 @@ static gpointer parent_class = NULL;
 
 
 struct _GthFileToolCropPrivate {
-	GdkPixbuf        *src_pixbuf;
 	GtkBuilder       *builder;
 	int               pixbuf_width;
 	int               pixbuf_height;
@@ -77,28 +76,32 @@ static void
 crop_button_clicked_cb (GtkButton       *button,
 			GthFileToolCrop *self)
 {
-	GdkRectangle  selection;
-	GdkPixbuf    *new_pixbuf;
+	GdkRectangle     selection;
+	GtkWidget       *window;
+	GtkWidget       *viewer_page;
+	GtkWidget       *viewer;
+	cairo_surface_t *old_image;
+	cairo_surface_t *new_image;
 
 	gth_image_selector_get_selection (self->priv->selector, &selection);
 	if ((selection.width == 0) || (selection.height == 0))
 		return;
 
-	new_pixbuf = gdk_pixbuf_new_subpixbuf (self->priv->src_pixbuf,
-					       selection.x,
-					       selection.y,
-					       selection.width,
-					       selection.height);
-	if (new_pixbuf != NULL) {
-		GtkWidget *window;
-		GtkWidget *viewer_page;
+	window = gth_file_tool_get_window (GTH_FILE_TOOL (self));
+	viewer_page = gth_browser_get_viewer_page (GTH_BROWSER (window));
+	viewer = gth_image_viewer_page_get_image_viewer (GTH_IMAGE_VIEWER_PAGE (viewer_page));
+	old_image = gth_image_viewer_get_current_image (GTH_IMAGE_VIEWER (viewer));
 
-		window = gth_file_tool_get_window (GTH_FILE_TOOL (self));
-		viewer_page = gth_browser_get_viewer_page (GTH_BROWSER (window));
-		gth_image_viewer_page_set_pixbuf (GTH_IMAGE_VIEWER_PAGE (viewer_page), new_pixbuf, TRUE);
+	new_image = _cairo_image_surface_copy_subsurface (old_image,
+					       	          selection.x,
+					       	          selection.y,
+					       	          selection.width,
+					       	          selection.height);
+	if (new_image != NULL) {
+		gth_image_viewer_page_set_image (GTH_IMAGE_VIEWER_PAGE (viewer_page), new_image, TRUE);
 		gth_file_tool_hide_options (GTH_FILE_TOOL (self));
 
-		g_object_unref (new_pixbuf);
+		cairo_surface_destroy (new_image);
 	}
 }
 
@@ -357,6 +360,29 @@ bind_factor_spinbutton_value_changed_cb (GtkSpinButton *spinbutton,
 }
 
 
+static void
+maximize_button_clicked_cb (GtkButton *button,
+			    gpointer   user_data)
+{
+	GthFileToolCrop *self = user_data;
+
+	gth_image_selector_set_selection_pos (self->priv->selector, 0, 0);
+	if (! gth_image_selector_set_selection_width (self->priv->selector, self->priv->pixbuf_width) || ! gth_image_selector_get_use_ratio (self->priv->selector))
+		gth_image_selector_set_selection_height (self->priv->selector, self->priv->pixbuf_height);
+	gth_image_selector_center (self->priv->selector);
+}
+
+
+static void
+center_button_clicked_cb (GtkButton *button,
+			  gpointer   user_data)
+{
+	GthFileToolCrop *self = user_data;
+
+	gth_image_selector_center (self->priv->selector);
+}
+
+
 static GtkWidget *
 gth_file_tool_crop_get_options (GthFileTool *base)
 {
@@ -364,6 +390,7 @@ gth_file_tool_crop_get_options (GthFileTool *base)
 	GtkWidget       *window;
 	GtkWidget       *viewer_page;
 	GtkWidget       *viewer;
+	cairo_surface_t *image;
 	GtkWidget       *options;
 	char            *text;
 
@@ -374,17 +401,13 @@ gth_file_tool_crop_get_options (GthFileTool *base)
 	if (! GTH_IS_IMAGE_VIEWER_PAGE (viewer_page))
 		return NULL;
 
-	_g_object_unref (self->priv->src_pixbuf);
-
 	viewer = gth_image_viewer_page_get_image_viewer (GTH_IMAGE_VIEWER_PAGE (viewer_page));
-	self->priv->src_pixbuf = gth_image_viewer_get_current_pixbuf (GTH_IMAGE_VIEWER (viewer));
-	if (self->priv->src_pixbuf == NULL)
+	image = gth_image_viewer_get_current_image (GTH_IMAGE_VIEWER (viewer));
+	if (image == NULL)
 		return NULL;
 
-	g_object_ref (self->priv->src_pixbuf);
-
-	self->priv->pixbuf_width = gdk_pixbuf_get_width (self->priv->src_pixbuf);
-	self->priv->pixbuf_height = gdk_pixbuf_get_height (self->priv->src_pixbuf);
+	self->priv->pixbuf_width = cairo_image_surface_get_width (image);
+	self->priv->pixbuf_height = cairo_image_surface_get_height (image);
 	_gtk_widget_get_screen_size (window, &self->priv->screen_width, &self->priv->screen_height);
 
 	self->priv->builder = _gtk_builder_new_from_file ("crop-options.ui", "file_tools");
@@ -398,12 +421,12 @@ gth_file_tool_crop_get_options (GthFileTool *base)
 
 	self->priv->ratio_combobox = _gtk_combo_box_new_with_texts (_("None"), _("Square"), NULL);
 	text = g_strdup_printf (_("%d x %d (Image)"), self->priv->pixbuf_width, self->priv->pixbuf_height);
-	gtk_combo_box_append_text (GTK_COMBO_BOX (self->priv->ratio_combobox), text);
+	gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (self->priv->ratio_combobox), text);
 	g_free (text);
 	text = g_strdup_printf (_("%d x %d (Screen)"), self->priv->screen_width, self->priv->screen_height);
-	gtk_combo_box_append_text (GTK_COMBO_BOX (self->priv->ratio_combobox), text);
+	gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (self->priv->ratio_combobox), text);
 	g_free (text);
-	_gtk_combo_box_append_texts (GTK_COMBO_BOX (self->priv->ratio_combobox),
+	_gtk_combo_box_append_texts (GTK_COMBO_BOX_TEXT (self->priv->ratio_combobox),
 				     _("5:4"),
 				     _("4:3 (DVD, Book)"),
 				     _("7:5"),
@@ -422,7 +445,12 @@ gth_file_tool_crop_get_options (GthFileTool *base)
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (GET_WIDGET ("ratio_h_spinbutton")), MAX (eel_gconf_get_integer (PREF_CROP_ASPECT_RATIO_HEIGHT, 1), 1));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("invert_ratio_checkbutton")), eel_gconf_get_boolean (PREF_CROP_ASPECT_RATIO_INVERT, FALSE));
 
-	self->priv->grid_type_combobox = _gtk_combo_box_new_with_texts (_("None"), _("Rule of Thirds"), _("Golden Sections"), NULL);
+	self->priv->grid_type_combobox = _gtk_combo_box_new_with_texts (_("None"),
+									_("Rule of Thirds"),
+									_("Golden Sections"),
+									_("Center Lines"),
+									_("Uniform"),
+									NULL);
 	gtk_combo_box_set_active (GTK_COMBO_BOX (self->priv->grid_type_combobox), eel_gconf_get_enum (PREF_CROP_GRID_TYPE, GTH_TYPE_GRID_TYPE, GTH_GRID_THIRDS));
 	gtk_widget_show (self->priv->grid_type_combobox);
 	gtk_box_pack_start (GTK_BOX (GET_WIDGET ("grid_type_combobox_box")), self->priv->grid_type_combobox, FALSE, FALSE, 0);
@@ -482,8 +510,16 @@ gth_file_tool_crop_get_options (GthFileTool *base)
 			  "value-changed",
 			  G_CALLBACK (bind_factor_spinbutton_value_changed_cb),
 			  self);
+	g_signal_connect (GET_WIDGET ("maximize_button"),
+			  "clicked",
+			  G_CALLBACK (maximize_button_clicked_cb),
+			  self);
+	g_signal_connect (GET_WIDGET ("center_button"),
+			  "clicked",
+			  G_CALLBACK (center_button_clicked_cb),
+			  self);
 
-	self->priv->selector = (GthImageSelector *) gth_image_selector_new (GTH_IMAGE_VIEWER (viewer), GTH_SELECTOR_TYPE_REGION);
+	self->priv->selector = (GthImageSelector *) gth_image_selector_new (GTH_SELECTOR_TYPE_REGION);
 	gth_image_selector_set_grid_type (self->priv->selector, gtk_combo_box_get_active (GTK_COMBO_BOX (self->priv->grid_type_combobox)));
 	gth_image_selector_bind_dimensions (self->priv->selector,
 					    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GET_WIDGET("bind_dimensions_checkbutton"))),
@@ -494,7 +530,14 @@ gth_file_tool_crop_get_options (GthFileTool *base)
 			  self);
 
 	gth_image_viewer_set_tool (GTH_IMAGE_VIEWER (viewer), (GthImageViewerTool *) self->priv->selector);
+	gth_image_viewer_set_zoom_quality (GTH_IMAGE_VIEWER (viewer), GTH_ZOOM_QUALITY_LOW);
+	gth_image_viewer_set_fit_mode (GTH_IMAGE_VIEWER (viewer), GTH_FIT_SIZE_IF_LARGER);
 	ratio_combobox_changed_cb (NULL, self);
+
+	if (! gth_image_selector_set_selection_width (self->priv->selector, self->priv->pixbuf_width * 2 / 3) || ! gth_image_selector_get_use_ratio (self->priv->selector))
+		gth_image_selector_set_selection_height (self->priv->selector, self->priv->pixbuf_height * 2 / 3);
+	gth_image_selector_center (self->priv->selector);
+
 	update_sensitivity (self);
 
 	return options;
@@ -524,10 +567,8 @@ gth_file_tool_crop_destroy_options (GthFileTool *base)
 
 		/* destroy the option data */
 
-		_g_object_unref (self->priv->src_pixbuf);
 		_g_object_unref (self->priv->builder);
 		_g_object_unref (self->priv->selector);
-		self->priv->src_pixbuf = NULL;
 		self->priv->builder = NULL;
 		self->priv->selector = NULL;
 	}
@@ -536,6 +577,7 @@ gth_file_tool_crop_destroy_options (GthFileTool *base)
 	viewer_page = gth_browser_get_viewer_page (GTH_BROWSER (window));
 	viewer = gth_image_viewer_page_get_image_viewer (GTH_IMAGE_VIEWER_PAGE (viewer_page));
 	gth_image_viewer_set_tool (GTH_IMAGE_VIEWER (viewer), NULL);
+	gth_image_viewer_set_zoom_quality (GTH_IMAGE_VIEWER (viewer), eel_gconf_get_enum (PREF_ZOOM_QUALITY, GTH_TYPE_ZOOM_QUALITY, GTH_ZOOM_QUALITY_HIGH));
 }
 
 
@@ -564,7 +606,6 @@ gth_file_tool_crop_finalize (GObject *object)
 
 	self = (GthFileToolCrop *) object;
 
-	_g_object_unref (self->priv->src_pixbuf);
 	_g_object_unref (self->priv->selector);
 	_g_object_unref (self->priv->builder);
 
