@@ -44,9 +44,10 @@ struct _GthImageViewerPagePrivate {
 	guint              cnxn_id[GCONF_NOTIFICATIONS];
 	guint              hide_mouse_timeout;
 	guint              motion_signal;
-	gboolean           pixbuf_changed;
+	gboolean           image_changed;
 	gboolean           shrink_wrap;
 	GFile             *last_loaded;
+	gboolean           can_paste;
 };
 
 static gpointer gth_image_viewer_page_parent_class = NULL;
@@ -58,6 +59,9 @@ static const char *image_viewer_ui_info =
 "      <placeholder name='File_Actions_1'>"
 "        <menuitem action='ImageViewer_Edit_Undo'/>"
 "        <menuitem action='ImageViewer_Edit_Redo'/>"
+"        <separator />"
+"        <menuitem action='ImageViewer_Edit_Copy_Image'/>"
+"        <menuitem action='ImageViewer_Edit_Paste_Image'/>"
 "      </placeholder>"
 "    </menu>"
 "    <menu name='View' action='ViewMenu'>"
@@ -149,6 +153,22 @@ image_viewer_activate_action_edit_redo (GtkAction          *action,
 }
 
 
+static void
+image_viewer_activate_action_edit_copy_image (GtkAction          *action,
+					      GthImageViewerPage *self)
+{
+	gth_image_viewer_page_copy_image (self);
+}
+
+
+static void
+image_viewer_activate_action_edit_paste_image (GtkAction          *action,
+					       GthImageViewerPage *self)
+{
+	gth_image_viewer_page_paste_image (self);
+}
+
+
 static void
 image_viewer_activate_action_view_shrink_wrap (GtkAction          *action,
 					       GthImageViewerPage *self)
@@ -168,6 +188,16 @@ static GtkActionEntry image_viewer_action_entries[] = {
 	  NULL,
 	  G_CALLBACK (image_viewer_activate_action_edit_redo) },
 
+	{ "ImageViewer_Edit_Copy_Image", GTK_STOCK_COPY,
+	  N_("Copy Image"), "<control>c",
+	  N_("Copy the image to the clipboard"),
+	  G_CALLBACK (image_viewer_activate_action_edit_copy_image) },
+
+	{ "ImageViewer_Edit_Paste_Image", GTK_STOCK_PASTE,
+	  N_("Paste Image"), "<control>p",
+	  N_("Paste the image from the clipboard"),
+	  G_CALLBACK (image_viewer_activate_action_edit_paste_image) },
+
 	{ "ImageViewer_View_ZoomIn", GTK_STOCK_ZOOM_IN,
 	  N_("In"), "<control>plus",
 	  N_("Zoom in"),
@@ -285,10 +315,97 @@ viewer_key_press_cb (GtkWidget          *widget,
 }
 
 
+static void
+_set_action_sensitive (GthImageViewerPage *self,
+		       const char         *action_name,
+		       gboolean            sensitive)
+{
+	GtkAction *action;
+
+	if (self->priv->actions == NULL)
+		return;
+	action = gtk_action_group_get_action (self->priv->actions, action_name);
+	g_object_set (action, "sensitive", sensitive, NULL);
+}
+
+
+static void
+clipboard_targets_received_cb (GtkClipboard *clipboard,
+			       GdkAtom      *atoms,
+                               int           n_atoms,
+                               gpointer      user_data)
+{
+	GthImageViewerPage *self = user_data;
+	int                 i;
+
+	self->priv->can_paste = FALSE;
+	for (i = 0; ! self->priv->can_paste && (i < n_atoms); i++)
+		if (atoms[i] == gdk_atom_intern_static_string ("image/png"))
+			self->priv->can_paste = TRUE;
+
+	_set_action_sensitive (self, "ImageViewer_Edit_Paste_Image", self->priv->can_paste);
+
+	g_object_unref (self);
+}
+
+
+static void
+_gth_image_viewer_page_update_paste_command_sensitivity (GthImageViewerPage *self,
+							 GtkClipboard       *clipboard)
+{
+	self->priv->can_paste = FALSE;
+        _set_action_sensitive (self, "ImageViewer_Edit_Paste_Image", FALSE);
+
+	if (clipboard == NULL)
+		clipboard = gtk_widget_get_clipboard (GTK_WIDGET (self->priv->viewer), GDK_SELECTION_CLIPBOARD);
+	gtk_clipboard_request_targets (clipboard,
+				       clipboard_targets_received_cb,
+				       g_object_ref (self));
+}
+
+
+static void
+clipboard_owner_change_cb (GtkClipboard *clipboard,
+                           GdkEvent     *event,
+                           gpointer      user_data)
+{
+	_gth_image_viewer_page_update_paste_command_sensitivity ((GthImageViewerPage *) user_data, clipboard);
+}
+
+
+static void
+viewer_realize_cb (GtkWidget *widget,
+                   gpointer   user_data)
+{
+	GthImageViewerPage *self = user_data;
+	GtkClipboard       *clipboard;
+
+	clipboard = gtk_widget_get_clipboard (self->priv->viewer, GDK_SELECTION_CLIPBOARD);
+	g_signal_connect (clipboard,
+	                  "owner_change",
+	                  G_CALLBACK (clipboard_owner_change_cb),
+	                  self);
+}
+
+
+static void
+viewer_unrealize_cb (GtkWidget *widget,
+		     gpointer   user_data)
+{
+	GthImageViewerPage *self = user_data;
+	GtkClipboard       *clipboard;
+
+	clipboard = gtk_widget_get_clipboard (self->priv->viewer, GDK_SELECTION_CLIPBOARD);
+	g_signal_handlers_disconnect_by_func (clipboard,
+	                                      G_CALLBACK (clipboard_owner_change_cb),
+	                                      self);
+}
+
+
 static void
 image_preloader_requested_ready_cb (GthImagePreloader  *preloader,
 				    GthFileData        *requested,
-				    GdkPixbufAnimation *animation,
+				    GthImage           *image,
 				    int                 original_width,
 				    int                 original_height,
 				    GError             *error,
@@ -303,15 +420,17 @@ image_preloader_requested_ready_cb (GthImagePreloader  *preloader,
 	}
 
 	gth_viewer_page_focus (GTH_VIEWER_PAGE (self));
-	gth_image_viewer_set_animation (GTH_IMAGE_VIEWER (self->priv->viewer),
-					animation,
-					original_width,
-					original_height);
+
+	gth_image_viewer_set_image (GTH_IMAGE_VIEWER (self->priv->viewer),
+				    image,
+				    original_width,
+				    original_height);
+
 	if (self->priv->shrink_wrap)
 		gth_image_viewer_page_shrink_wrap (self, TRUE);
 	gth_image_history_clear (self->priv->history);
 	gth_image_history_add_image (self->priv->history,
-				     gth_image_viewer_get_current_pixbuf (GTH_IMAGE_VIEWER (self->priv->viewer)),
+				     gth_image_viewer_get_current_image (GTH_IMAGE_VIEWER (self->priv->viewer)),
 				     FALSE);
 	gth_image_viewer_page_file_loaded (self, TRUE);
 }
@@ -320,7 +439,7 @@ image_preloader_requested_ready_cb (GthImagePreloader  *preloader,
 static void
 image_preloader_original_size_ready_cb (GthImagePreloader  *preloader,
 				        GthFileData        *requested,
-				        GdkPixbufAnimation *animation,
+				        GthImage           *image,
 				        int                 original_width,
 				        int                 original_height,
 				        GError             *error,
@@ -333,12 +452,12 @@ image_preloader_original_size_ready_cb (GthImagePreloader  *preloader,
 		return;
 
 	gth_image_viewer_set_better_quality (GTH_IMAGE_VIEWER (self->priv->viewer),
-					     animation,
+					     image,
 					     original_width,
 					     original_height);
 	gth_image_history_clear (self->priv->history);
 	gth_image_history_add_image (self->priv->history,
-				     gth_image_viewer_get_current_pixbuf (GTH_IMAGE_VIEWER (self->priv->viewer)),
+				     gth_image_viewer_get_current_image (GTH_IMAGE_VIEWER (self->priv->viewer)),
 				     FALSE);
 }
 
@@ -451,6 +570,153 @@ pref_viewer_shrink_wrap_changed (GConfClient *client,
 }
 
 
+static void
+paint_comment_over_image_func (GthImageViewer *image_viewer,
+			       GdkEventExpose *event,
+			       cairo_t        *cr,
+			       gpointer        user_data)
+{
+	GthImageViewerPage *self = user_data;
+	GthFileData        *file_data = self->priv->file_data;
+	GString            *file_info;
+	char               *comment;
+	const char         *file_date;
+	const char         *file_size;
+	int                 current_position;
+	int                 n_visibles;
+	int                 width;
+	int                 height;
+	GthMetadata        *metadata;
+	PangoLayout        *layout;
+	PangoAttrList      *attr_list = NULL;
+	GError             *error = NULL;
+	char               *text;
+	static GdkPixbuf   *icon = NULL;
+	int                 icon_width;
+	int                 icon_height;
+	int                 image_width;
+	int                 image_height;
+	const int           x_padding = 10;
+	const int           y_padding = 10;
+	int                 max_text_width;
+	PangoRectangle      bounds;
+	int                 text_x;
+	int                 text_y;
+	int                 icon_x;
+	int                 icon_y;
+
+	file_info = g_string_new ("");
+
+	comment = gth_file_data_get_attribute_as_string (file_data, "general::description");
+	if (comment != NULL) {
+		g_string_append_printf (file_info, "<b>%s</b>\n\n", comment);
+		g_free (comment);
+	}
+
+	metadata = (GthMetadata *) g_file_info_get_attribute_object (file_data->info, "general::datetime");
+	if (metadata != NULL)
+		file_date = gth_metadata_get_formatted (metadata);
+	else
+		file_date = g_file_info_get_attribute_string (file_data->info, "gth::file::display-mtime");
+	file_size = g_file_info_get_attribute_string (file_data->info, "gth::file::display-size");
+
+	gth_browser_get_file_list_info (self->priv->browser, &current_position, &n_visibles);
+	gth_image_viewer_get_original_size (GTH_IMAGE_VIEWER (self->priv->viewer), &width, &height);
+
+	g_string_append_printf (file_info,
+			        "<small><i>%s - %dx%d (%d%%) - %s</i>\n<tt>%d/%d - %s</tt></small>",
+			        file_date,
+			        width,
+			        height,
+			        (int) (gth_image_viewer_get_zoom (GTH_IMAGE_VIEWER (self->priv->viewer)) * 100),
+			        file_size,
+			        current_position + 1,
+			        n_visibles,
+				g_file_info_get_attribute_string (file_data->info, "standard::display-name"));
+
+	layout = gtk_widget_create_pango_layout (GTK_WIDGET (self->priv->viewer), NULL);
+	pango_layout_set_wrap (layout, PANGO_WRAP_WORD);
+	pango_layout_set_alignment (layout, PANGO_ALIGN_LEFT);
+
+	if (! pango_parse_markup (file_info->str,
+			          -1,
+				  0,
+				  &attr_list,
+				  &text,
+				  NULL,
+				  &error))
+	{
+		g_warning ("Failed to set text from markup due to error parsing markup: %s\nThis is the text that caused the error: %s",  error->message, file_info->str);
+		g_error_free (error);
+		g_object_unref (layout);
+		g_string_free (file_info, TRUE);
+		return;
+	}
+
+	pango_layout_set_attributes (layout, attr_list);
+        pango_layout_set_text (layout, text, strlen (text));
+
+        if (icon == NULL) {
+        	GIcon *gicon;
+
+        	gicon = g_themed_icon_new (GTK_STOCK_PROPERTIES);
+        	icon = _g_icon_get_pixbuf (gicon, 24, NULL);
+
+        	g_object_unref (gicon);
+        }
+	icon_width = gdk_pixbuf_get_width (icon);
+	icon_height = gdk_pixbuf_get_height (icon);
+
+	image_width = gdk_window_get_width (gtk_widget_get_window (self->priv->viewer));
+	image_height = gdk_window_get_height (gtk_widget_get_window (self->priv->viewer));
+	max_text_width = ((image_width * 3 / 4) - icon_width - (x_padding * 3) - (x_padding * 2));
+
+	pango_layout_set_width (layout, max_text_width * PANGO_SCALE);
+	pango_layout_get_pixel_extents (layout, NULL, &bounds);
+
+	bounds.width += (2 * x_padding) + (icon_width + x_padding);
+	bounds.height = MIN (image_height - icon_height - (y_padding * 2), bounds.height + (2 * y_padding));
+	bounds.x = MAX ((image_width - bounds.width) / 2, 0);
+	bounds.y = MAX (image_height - bounds.height - (y_padding * 3), 0);
+
+	text_x = bounds.x + x_padding + icon_width + x_padding;
+	text_y = bounds.y + y_padding;
+	icon_x = bounds.x + x_padding;
+	icon_y = bounds.y + (bounds.height - icon_height) / 2;
+
+	cairo_save (cr);
+
+	/* background */
+
+	_cairo_draw_rounded_box (cr, bounds.x, bounds.y, bounds.width, bounds.height, 8.0);
+	cairo_set_source_rgba (cr, 0.94, 0.94, 0.94, 0.81);
+	cairo_fill (cr);
+	cairo_set_line_width (cr, 1.0);
+	cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
+	cairo_stroke (cr);
+
+	/* icon */
+
+	gdk_cairo_set_source_pixbuf (cr, icon, icon_x, icon_y);
+	cairo_rectangle (cr, icon_x, icon_y, icon_width, icon_height);
+	cairo_fill (cr);
+
+	/* text */
+
+	cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
+	pango_cairo_update_layout (cr, layout);
+	cairo_move_to (cr, text_x, text_y);
+	pango_cairo_show_layout (cr, layout);
+
+	cairo_restore (cr);
+
+        g_free (text);
+        pango_attr_list_unref (attr_list);
+	g_object_unref (layout);
+	g_string_free (file_info, TRUE);
+}
+
+
 static void
 gth_image_viewer_page_real_activate (GthViewerPage *base,
 				     GthBrowser    *browser)
@@ -525,6 +791,14 @@ gth_image_viewer_page_real_activate (GthViewerPage *base,
 			  "key_press_event",
 			  G_CALLBACK (viewer_key_press_cb),
 			  self);
+	g_signal_connect (G_OBJECT (self->priv->viewer),
+			  "realize",
+			  G_CALLBACK (viewer_realize_cb),
+			  self);
+	g_signal_connect (G_OBJECT (self->priv->viewer),
+			  "unrealize",
+			  G_CALLBACK (viewer_unrealize_cb),
+			  self);
 
 	self->priv->image_navigator = gth_image_navigator_new (GTH_IMAGE_VIEWER (self->priv->viewer));
 	gtk_widget_show (self->priv->image_navigator);
@@ -651,15 +925,15 @@ static gboolean
 gth_image_viewer_page_real_can_view (GthViewerPage *base,
 				     GthFileData   *file_data)
 {
-	GthImageViewerPage *self;
-
-	self = (GthImageViewerPage*) base;
 	g_return_val_if_fail (file_data != NULL, FALSE);
 
 	return _g_mime_type_is_image (gth_file_data_get_mime_type (file_data));
 }
 
 
+#define N_PRELOADERS 4
+
+
 static void
 gth_image_viewer_page_real_view (GthViewerPage *base,
 				 GthFileData   *file_data)
@@ -667,9 +941,9 @@ gth_image_viewer_page_real_view (GthViewerPage *base,
 	GthImageViewerPage *self;
 	GthFileStore       *file_store;
 	GtkTreeIter         iter;
-	GthFileData        *next_file_data = NULL;
-	GthFileData        *next2_file_data = NULL;
-	GthFileData        *prev_file_data = NULL;
+	int                 i;
+	GthFileData        *next_file_data[N_PRELOADERS];
+	GthFileData        *prev_file_data[N_PRELOADERS];
 	int                 window_width;
 	int                 window_height;
 
@@ -683,7 +957,7 @@ gth_image_viewer_page_real_view (GthViewerPage *base,
 	if ((self->priv->file_data != NULL)
 	    && g_file_equal (file_data->file, self->priv->file_data->file)
 	    && (gth_file_data_get_mtime (file_data) == gth_file_data_get_mtime (self->priv->file_data))
-	    && ! self->priv->pixbuf_changed)
+	    && ! self->priv->image_changed)
 	{
 		gth_image_viewer_page_file_loaded (self, TRUE);
 		return;
@@ -691,23 +965,30 @@ gth_image_viewer_page_real_view (GthViewerPage *base,
 
 	_g_object_unref (self->priv->file_data);
 	self->priv->file_data = gth_file_data_dup (file_data);
+	self->priv->image_changed = FALSE;
+
+	for (i = 0; i < N_PRELOADERS; i++) {
+		next_file_data[i] = NULL;
+		prev_file_data[i] = NULL;
+	}
 
 	file_store = gth_browser_get_file_store (self->priv->browser);
 	if (gth_file_store_find_visible (file_store, self->priv->file_data->file, &iter)) {
-		GtkTreeIter iter2;
-		GtkTreeIter iter3;
+		GtkTreeIter next_iter;
 
-		iter2 = iter;
-		if (gth_file_store_get_next_visible (file_store, &iter2))
-			next_file_data = gth_file_store_get_file (file_store, &iter2);
+		next_iter = iter;
+		for (i = 0; i < N_PRELOADERS; i++) {
+			if (! gth_file_store_get_next_visible (file_store, &next_iter))
+				break;
+			next_file_data[i] = gth_file_store_get_file (file_store, &next_iter);
+		}
 
-		iter3 = iter2;
-		if (gth_file_store_get_next_visible (file_store, &iter3))
-			next2_file_data = gth_file_store_get_file (file_store, &iter3);
-
-		iter2 = iter;
-		if (gth_file_store_get_prev_visible (file_store, &iter2))
-			prev_file_data = gth_file_store_get_file (file_store, &iter2);
+		next_iter = iter;
+		for (i = 0; i < N_PRELOADERS; i++) {
+			if (! gth_file_store_get_prev_visible (file_store, &next_iter))
+				break;
+			prev_file_data[i] = gth_file_store_get_file (file_store, &next_iter);
+		}
 	}
 
 	gtk_window_get_size (GTK_WINDOW (self->priv->browser),
@@ -717,9 +998,14 @@ gth_image_viewer_page_real_view (GthViewerPage *base,
 	gth_image_preloader_load (self->priv->preloader,
 				  self->priv->file_data,
 				  (gth_image_prelaoder_get_load_policy (self->priv->preloader) == GTH_LOAD_POLICY_TWO_STEPS) ? MAX (window_width, window_height) : -1,
-				  next_file_data,
-				  next2_file_data,
-				  prev_file_data,
+				  next_file_data[0],
+				  next_file_data[1],
+				  next_file_data[2],
+				  next_file_data[3],
+				  prev_file_data[0],
+				  prev_file_data[1],
+				  prev_file_data[2],
+				  prev_file_data[3],
 				  NULL);
 }
 
@@ -742,6 +1028,7 @@ gth_image_viewer_page_real_fullscreen (GthViewerPage *base,
 	GthImageViewerPage *self;
 
 	self = (GthImageViewerPage *) base;
+
 	if (active) {
 		gth_image_navigator_set_scrollbars_visible (GTH_IMAGE_NAVIGATOR (self->priv->image_navigator), FALSE);
 		gth_image_viewer_set_black_background (GTH_IMAGE_VIEWER (self->priv->viewer), TRUE);
@@ -768,22 +1055,11 @@ gth_image_viewer_page_real_show_pointer (GthViewerPage *base,
 }
 
 
-static void
-_set_action_sensitive (GthImageViewerPage *self,
-		       const char         *action_name,
-		       gboolean            sensitive)
-{
-	GtkAction *action;
-
-	action = gtk_action_group_get_action (self->priv->actions, action_name);
-	g_object_set (action, "sensitive", sensitive, NULL);
-}
-
-
 static void
 gth_image_viewer_page_real_update_sensitivity (GthViewerPage *base)
 {
 	GthImageViewerPage *self;
+	gboolean            zoom_enabled;
 	double              zoom;
 	GthFit              fit_mode;
 
@@ -792,14 +1068,18 @@ gth_image_viewer_page_real_update_sensitivity (GthViewerPage *base)
 	_set_action_sensitive (self, "ImageViewer_Edit_Undo", gth_image_history_can_undo (self->priv->history));
 	_set_action_sensitive (self, "ImageViewer_Edit_Redo", gth_image_history_can_redo (self->priv->history));
 
+	zoom_enabled = gth_image_viewer_get_zoom_enabled (GTH_IMAGE_VIEWER (self->priv->viewer));
 	zoom = gth_image_viewer_get_zoom (GTH_IMAGE_VIEWER (self->priv->viewer));
-	_set_action_sensitive (self, "ImageViewer_View_Zoom100", ! FLOAT_EQUAL (zoom, 1.0));
-	_set_action_sensitive (self, "ImageViewer_View_ZoomOut", zoom > 0.05);
-	_set_action_sensitive (self, "ImageViewer_View_ZoomIn", zoom < 100.0);
+
+	_set_action_sensitive (self, "ImageViewer_View_Zoom100", zoom_enabled && ! FLOAT_EQUAL (zoom, 1.0));
+	_set_action_sensitive (self, "ImageViewer_View_ZoomOut", zoom_enabled && (zoom > 0.05));
+	_set_action_sensitive (self, "ImageViewer_View_ZoomIn", zoom_enabled && (zoom < 100.0));
 
 	fit_mode = gth_image_viewer_get_fit_mode (GTH_IMAGE_VIEWER (self->priv->viewer));
-	_set_action_sensitive (self, "ImageViewer_View_ZoomFit", fit_mode != GTH_FIT_SIZE);
-	_set_action_sensitive (self, "ImageViewer_View_ZoomFitWidth", fit_mode != GTH_FIT_WIDTH);
+	_set_action_sensitive (self, "ImageViewer_View_ZoomFit", zoom_enabled && (fit_mode != GTH_FIT_SIZE));
+	_set_action_sensitive (self, "ImageViewer_View_ZoomFitWidth", zoom_enabled && (fit_mode != GTH_FIT_WIDTH));
+
+	_gth_image_viewer_page_update_paste_command_sensitivity (self, NULL);
 }
 
 
@@ -835,7 +1115,7 @@ image_saved_cb (GthFileData *file_data,
 	if (data->func != NULL)
 		(data->func) ((GthViewerPage *) self, self->priv->file_data, error, data->user_data);
 	else if (error != NULL)
-		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (self->priv->browser), _("Could not save the file"), &error);
+		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (self->priv->browser), _("Could not save the file"), error);
 
 	if (! error_occurred) {
 		GFile *folder;
@@ -867,6 +1147,7 @@ _gth_image_viewer_page_real_save (GthViewerPage *base,
 	GthImageViewerPage *self;
 	SaveData           *data;
 	GthFileData        *current_file;
+	GdkPixbuf          *pixbuf;
 
 	self = (GthImageViewerPage *) base;
 
@@ -884,12 +1165,15 @@ _gth_image_viewer_page_real_save (GthViewerPage *base,
 		gth_file_data_set_file (current_file, file);
 	g_file_info_set_attribute_boolean (current_file->info, "gth::file::is-modified", FALSE);
 
-	_gdk_pixbuf_save_async (gth_image_viewer_get_current_pixbuf (GTH_IMAGE_VIEWER (self->priv->viewer)),
+	pixbuf = gth_image_viewer_get_current_pixbuf (GTH_IMAGE_VIEWER (self->priv->viewer));
+	_gdk_pixbuf_save_async (pixbuf,
 			        current_file,
 			        mime_type,
 			        TRUE,
 				image_saved_cb,
 				data);
+
+	_g_object_unref (pixbuf);
 }
 
 
@@ -1011,23 +1295,23 @@ gth_image_viewer_page_real_save_as (GthViewerPage *base,
 
 
 static void
-_gth_image_viewer_page_set_pixbuf (GthImageViewerPage *self,
-				   GdkPixbuf          *pixbuf,
-				   gboolean            modified)
+_gth_image_viewer_page_set_image (GthImageViewerPage *self,
+				  cairo_surface_t    *image,
+				  gboolean            modified)
 {
 	GthFileData *file_data;
 	int          width;
 	int          height;
 	char        *size;
 
-	gth_image_viewer_set_pixbuf (GTH_IMAGE_VIEWER (self->priv->viewer), pixbuf, -1, -1);
+	gth_image_viewer_set_surface (GTH_IMAGE_VIEWER (self->priv->viewer), image, -1, -1);
 
 	file_data = gth_browser_get_current_file (GTH_BROWSER (self->priv->browser));
 
 	g_file_info_set_attribute_boolean (file_data->info, "gth::file::is-modified", modified);
 
-	width = gdk_pixbuf_get_width (pixbuf);
-	height = gdk_pixbuf_get_height (pixbuf);
+	width = cairo_image_surface_get_width (image);
+	height = cairo_image_surface_get_height (image);
 	g_file_info_set_attribute_int32 (file_data->info, "image::width", width);
 	g_file_info_set_attribute_int32 (file_data->info, "image::height", height);
 
@@ -1048,12 +1332,46 @@ gth_image_viewer_page_real_revert (GthViewerPage *base)
 
 	idata = gth_image_history_revert (self->priv->history);
 	if (idata != NULL) {
-		_gth_image_viewer_page_set_pixbuf (self, idata->image, idata->unsaved);
+		_gth_image_viewer_page_set_image (self, idata->image, idata->unsaved);
 		gth_image_data_unref (idata);
 	}
 }
 
 
+static void
+gth_image_viewer_page_real_update_info (GthViewerPage *base,
+					GthFileData   *file_data)
+{
+	GthImageViewerPage *self = GTH_IMAGE_VIEWER_PAGE (base);
+
+	if (! _g_file_equal (self->priv->file_data->file, file_data->file))
+		return;
+	_g_object_unref (self->priv->file_data);
+	self->priv->file_data = gth_file_data_dup (file_data);
+
+	if (self->priv->viewer == NULL)
+		return;
+
+	gth_image_viewer_update_view (GTH_IMAGE_VIEWER (self->priv->viewer));
+}
+
+
+static void
+gth_image_viewer_page_real_show_properties (GthViewerPage *base,
+					    gboolean       show)
+{
+	GthImageViewerPage *self;
+
+	self = GTH_IMAGE_VIEWER_PAGE (base);
+
+	if (show)
+		gth_image_viewer_add_painter (GTH_IMAGE_VIEWER (self->priv->viewer), paint_comment_over_image_func, self);
+	else
+		gth_image_viewer_remove_painter (GTH_IMAGE_VIEWER (self->priv->viewer), paint_comment_over_image_func, self);
+	gth_image_viewer_update_view (GTH_IMAGE_VIEWER (self->priv->viewer));
+}
+
+
 static void
 gth_image_viewer_page_finalize (GObject *obj)
 {
@@ -1096,6 +1414,8 @@ gth_viewer_page_interface_init (GthViewerPageIface *iface)
 	iface->save = gth_image_viewer_page_real_save;
 	iface->save_as = gth_image_viewer_page_real_save_as;
 	iface->revert = gth_image_viewer_page_real_revert;
+	iface->update_info = gth_image_viewer_page_real_update_info;
+	iface->show_properties = gth_image_viewer_page_real_show_properties;
 }
 
 
@@ -1106,6 +1426,8 @@ gth_image_viewer_page_instance_init (GthImageViewerPage *self)
 	self->priv->history = gth_image_history_new ();
 	self->priv->shrink_wrap = FALSE;
 	self->priv->last_loaded = NULL;
+	self->priv->image_changed = FALSE;
+	self->priv->can_paste = FALSE;
 }
 
 
@@ -1155,11 +1477,34 @@ void
 gth_image_viewer_page_set_pixbuf (GthImageViewerPage *self,
 				  GdkPixbuf          *pixbuf,
 				  gboolean            add_to_history)
+{
+	cairo_surface_t *image;
+
+	image = _cairo_image_surface_create_from_pixbuf (pixbuf);
+	gth_image_viewer_page_set_image (self, image, add_to_history);
+
+	cairo_surface_destroy (image);
+}
+
+
+cairo_surface_t *
+gth_image_viewer_page_get_image (GthImageViewerPage *self)
+{
+	return gth_image_viewer_get_current_image (GTH_IMAGE_VIEWER (self->priv->viewer));
+}
+
+
+void
+gth_image_viewer_page_set_image (GthImageViewerPage *self,
+			 	 cairo_surface_t    *image,
+			 	 gboolean            add_to_history)
 {
 	if (add_to_history)
-		gth_image_history_add_image (self->priv->history, pixbuf, TRUE);
-	_gth_image_viewer_page_set_pixbuf (self, pixbuf, TRUE);
-	self->priv->pixbuf_changed = TRUE;
+		gth_image_history_add_image (self->priv->history, image, TRUE);
+	_gth_image_viewer_page_set_image (self, image, TRUE);
+	self->priv->image_changed = TRUE;
+	if (add_to_history)
+		gth_viewer_page_focus (GTH_VIEWER_PAGE (self));
 }
 
 
@@ -1170,7 +1515,7 @@ gth_image_viewer_page_undo (GthImageViewerPage *self)
 
 	idata = gth_image_history_undo (self->priv->history);
 	if (idata != NULL)
-		_gth_image_viewer_page_set_pixbuf (self, idata->image, idata->unsaved);
+		_gth_image_viewer_page_set_image (self, idata->image, idata->unsaved);
 }
 
 
@@ -1181,7 +1526,7 @@ gth_image_viewer_page_redo (GthImageViewerPage *self)
 
 	idata = gth_image_history_redo (self->priv->history);
 	if (idata != NULL)
-		_gth_image_viewer_page_set_pixbuf (self, idata->image, idata->unsaved);
+		_gth_image_viewer_page_set_image (self, idata->image, idata->unsaved);
 }
 
 
@@ -1201,7 +1546,7 @@ gth_image_viewer_page_reset (GthImageViewerPage *self)
 	if (last_image == NULL)
 		return;
 
-	_gth_image_viewer_page_set_pixbuf (self, last_image->image, last_image->unsaved);
+	_gth_image_viewer_page_set_image (self, last_image->image, last_image->unsaved);
 }
 
 
@@ -1211,7 +1556,7 @@ add_non_content_width (GthImageViewerPage *self,
 {
 	int width = 0;
 
-	if ((non_content != NULL) && gtk_widget_get_visible (non_content)) {
+	if ((non_content != NULL) && gtk_widget_get_mapped (non_content)) {
 		GtkAllocation allocation;
 
 		gtk_widget_get_allocation (non_content, &allocation);
@@ -1228,7 +1573,7 @@ add_non_content_height (GthImageViewerPage *self,
 {
 	int height = 0;
 
-	if ((non_content != NULL) && gtk_widget_get_visible (non_content)) {
+	if ((non_content != NULL) && gtk_widget_get_mapped (non_content)) {
 		GtkAllocation allocation;
 
 		gtk_widget_get_allocation (non_content, &allocation);
@@ -1319,3 +1664,47 @@ gth_image_viewer_page_shrink_wrap (GthImageViewerPage *self,
 		gth_window_apply_saved_size (GTH_WINDOW (self->priv->browser), GTH_BROWSER_PAGE_VIEWER);
 	gth_image_viewer_set_fit_mode (GTH_IMAGE_VIEWER (self->priv->viewer), GTH_FIT_SIZE_IF_LARGER);
 }
+
+
+void
+gth_image_viewer_page_copy_image (GthImageViewerPage *self)
+{
+	cairo_surface_t *image;
+	GtkClipboard    *clipboard;
+	GdkPixbuf       *pixbuf;
+
+	image = gth_image_viewer_get_current_image (GTH_IMAGE_VIEWER (self->priv->viewer));
+	if (image == NULL)
+		return;
+
+	clipboard = gtk_clipboard_get_for_display (gtk_widget_get_display (self->priv->viewer), GDK_SELECTION_CLIPBOARD);
+	pixbuf = _gdk_pixbuf_new_from_cairo_surface (image);
+	gtk_clipboard_set_image (clipboard, pixbuf);
+
+	g_object_unref (pixbuf);
+}
+
+
+static void
+clipboard_image_received_cb (GtkClipboard *clipboard,
+			     GdkPixbuf    *pixbuf,
+			     gpointer      user_data)
+{
+	GthImageViewerPage *self = user_data;
+
+	if (pixbuf != NULL)
+		gth_image_viewer_page_set_pixbuf (self, pixbuf, TRUE);
+	g_object_unref (self);
+}
+
+
+void
+gth_image_viewer_page_paste_image (GthImageViewerPage *self)
+{
+	GtkClipboard *clipboard;
+
+	clipboard = gtk_clipboard_get_for_display (gtk_widget_get_display (self->priv->viewer), GDK_SELECTION_CLIPBOARD);
+	gtk_clipboard_request_image (clipboard,
+				     clipboard_image_received_cb,
+				     g_object_ref (self));
+}
