@@ -20,9 +20,12 @@
  */
 
 #include <config.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gthumb.h>
 #include "dlg-rename-series.h"
+#include "gth-template-editor-dialog.h"
+#include "gth-template-selector.h"
 #include "gth-rename-task.h"
 #include "preferences.h"
 
@@ -53,14 +56,29 @@ enum {
 #define DEFAULT_SORT_BY        "general::unsorted"
 #define DEFAULT_REVERSE_ORDER  FALSE
 #define DEFAULT_CHANGE_CASE    GTH_CHANGE_CASE_NONE
+#define UPDATE_DELAY           500
+
+
+static GthTemplateCode Rename_Special_Codes[] = {
+	{ GTH_TEMPLATE_CODE_TYPE_TEXT, N_("Text"), 0 },
+	{ GTH_TEMPLATE_CODE_TYPE_ENUMERATOR, N_("Enumerator"), '#' },
+	{ GTH_TEMPLATE_CODE_TYPE_SIMPLE, N_("Original filename"), 'F' },
+	{ GTH_TEMPLATE_CODE_TYPE_SIMPLE, N_("Original extension"), 'E' },
+	{ GTH_TEMPLATE_CODE_TYPE_SIMPLE, N_("Original enumerator"), 'N' },
+	{ GTH_TEMPLATE_CODE_TYPE_DATE, N_("Modification date"), 'M' },
+	{ GTH_TEMPLATE_CODE_TYPE_DATE, N_("Digitalization date"), 'D' },
+	{ GTH_TEMPLATE_CODE_TYPE_FILE_ATTRIBUTE, N_("File attribute"), 'A' }
+};
 
 
 typedef struct {
 	GthBrowser    *browser;
 	GList         *file_list;
+	GList         *file_data_list;
 	GList         *new_file_list;
 	GList         *new_names_list;
 	gboolean       single_file;
+	gboolean       first_update;
 	GtkBuilder    *builder;
 	GtkWidget     *dialog;
 	GtkWidget     *list_view;
@@ -69,16 +87,30 @@ typedef struct {
 	GtkListStore  *list_store;
 	GtkListStore  *sort_model;
 	gboolean       help_visible;
+	char          *required_attributes;
+	guint          update_id;
+	gboolean       template_changed;
+	GList         *tasks;
+	gboolean       closing;
 } DialogData;
 
 
 static void
-destroy_cb (GtkWidget  *widget,
-	    DialogData *data)
+destroy_dialog (DialogData *data)
 {
+	if (data->dialog != NULL)
+		gtk_widget_destroy (data->dialog);
+	data->dialog = NULL;
 	gth_browser_set_dialog (data->browser, "rename_series", NULL);
 
+	if (data->update_id != 0) {
+		g_source_remove (data->update_id);
+		data->update_id = 0;
+	}
+
+	g_free (data->required_attributes);
 	g_object_unref (data->builder);
+	_g_object_list_unref (data->file_data_list);
 	_g_object_list_unref (data->file_list);
 	_g_string_list_free (data->new_names_list);
 	g_list_free (data->new_file_list);
@@ -86,78 +118,6 @@ destroy_cb (GtkWidget  *widget,
 }
 
 
-static void
-ok_clicked_cb (GtkWidget  *widget,
-	       DialogData *data)
-{
-	GtkTreeIter   iter;
-	GList        *old_files;
-	GList        *new_files;
-	GList        *scan1;
-	GList        *scan2;
-	GthTask      *task;
-
-	/* -- save preferences -- */
-
-	if (data->file_list->next != NULL)
-		eel_gconf_set_string (PREF_RENAME_SERIES_TEMPLATE, gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("template_entry"))));
-
-	eel_gconf_set_integer (PREF_RENAME_SERIES_START_AT, gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (GET_WIDGET ("start_at_spinbutton"))));
-
-	if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (data->sort_combobox), &iter)) {
-		GthFileDataSort *sort_type;
-
-		gtk_tree_model_get (GTK_TREE_MODEL (data->sort_model),
-				    &iter,
-				    SORT_DATA_COLUMN, &sort_type,
-				    -1);
-		eel_gconf_set_string (PREF_RENAME_SERIES_SORT_BY, sort_type->name);
-	}
-
-	eel_gconf_set_boolean (PREF_RENAME_SERIES_REVERSE_ORDER, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("reverse_order_checkbutton"))));
-	eel_gconf_set_integer (PREF_RENAME_SERIES_CHANGE_CASE, gtk_combo_box_get_active (GTK_COMBO_BOX (data->change_case_combobox)));
-
-	/* -- prepare and exec rename task -- */
-
-	old_files = NULL;
-	new_files = NULL;
-
-	for (scan1 = data->new_file_list, scan2 = data->new_names_list;
-	     scan1 && scan2;
-	     scan1 = scan1->next, scan2 = scan2->next)
-	{
-		GthFileData *file_data = scan1->data;
-		char        *new_name  = scan2->data;
-		GFile       *parent;
-		GFile       *new_file;
-
-		parent = g_file_get_parent (file_data->file);
-		new_file = g_file_get_child (parent, new_name);
-
-		old_files = g_list_prepend (old_files, g_object_ref (file_data->file));
-		new_files = g_list_prepend (new_files, new_file);
-
-		g_object_unref (parent);
-	}
-	old_files = g_list_reverse (old_files);
-	new_files = g_list_reverse (new_files);
-
-	task = gth_rename_task_new (old_files, new_files);
-	gth_browser_exec_task (data->browser, task, FALSE);
-
-	g_object_unref (task);
-	gtk_widget_destroy (data->dialog);
-}
-
-
-static void
-help_clicked_cb (GtkWidget  *widget,
-                 DialogData *data)
-{
-        show_help_dialog (GTK_WINDOW (data->dialog), "gthumb-rename-series");
-}
-
-
 typedef struct {
 	const char   *template;
 	GthFileData  *file_data;
@@ -304,16 +264,129 @@ template_eval_cb (const GMatchInfo *info,
 }
 
 
-static void
-dlg_rename_series_update_preview (DialogData *data)
+static char *
+get_required_attributes (DialogData *data)
 {
+	GtkTreeIter  iter;
+	GString     *required_attributes;
+	const char  *template;
+
+	required_attributes = g_string_new (GFILE_STANDARD_ATTRIBUTES);
+
+	/* attributes required for sorting */
+
+	if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (data->sort_combobox), &iter)) {
+		GthFileDataSort *sort_type;
+
+		gtk_tree_model_get (GTK_TREE_MODEL (data->sort_model),
+				    &iter,
+				    SORT_DATA_COLUMN, &sort_type,
+				    -1);
+
+		if ((sort_type->required_attributes != NULL) && ! g_str_equal (sort_type->required_attributes, "")) {
+			g_string_append (required_attributes, ",");
+			g_string_append (required_attributes, sort_type->required_attributes);
+		}
+	}
+
+	/* attributes required for renaming */
+
+	template = gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("template_entry")));
+
+	if (g_strstr_len (template, -1, "%A") != NULL) {
+		GRegex  *re;
+		char   **a;
+		int      i;
+
+		re = g_regex_new ("%A\\{([^}]+)\\}", 0, 0, NULL);
+		a = g_regex_split (re, template, 0);
+		for (i = 1; i < g_strv_length (a); i += 2) {
+			char *id;
+
+			id = g_strstrip (g_strdup (a[i]));
+			g_string_append (required_attributes, ",");
+			g_string_append (required_attributes, id);
+
+			g_free (id);
+		}
+
+		g_strfreev (a);
+		g_regex_unref (re);
+	}
+
+	if (g_strstr_len (template, -1, "%D") != NULL) {
+		int i;
+
+		for (i = 0; FileDataDigitalizationTags[i] != NULL; i++) {
+			g_string_append (required_attributes, ",");
+			g_string_append (required_attributes, FileDataDigitalizationTags[i]);
+		}
+	}
+
+	if (g_strstr_len (template, -1, "%M") != NULL) {
+		g_string_append (required_attributes, ",");
+		g_string_append (required_attributes, G_FILE_ATTRIBUTE_TIME_MODIFIED "," G_FILE_ATTRIBUTE_TIME_MODIFIED_USEC);
+	}
+
+	return g_string_free (required_attributes, FALSE);
+}
+
+
+/* update_file_list */
+
+
+typedef struct {
+	DialogData *data;
+	ReadyFunc   ready_func;
+	GthTask    *task;
+	gulong      task_completed_id;
+} UpdateData;
+
+
+static void
+update_data_free (UpdateData *update_data)
+{
+	g_free(update_data);
+}
+
+
+static void update_preview_cb (GtkWidget  *widget, DialogData *data);
+
+
+static void
+update_file_list__step2 (gpointer user_data)
+{
+	UpdateData   *update_data = user_data;
+	DialogData   *data = update_data->data;
 	GtkTreeIter   iter;
 	int           change_case;
 	TemplateData *template_data;
 	GRegex       *re;
 	GList        *scan;
 	GError       *error = NULL;
-	GList        *scan1, *scan2;
+
+	if (data->first_update && (data->file_data_list->next == NULL)) {
+		GthFileData *file_data = data->file_data_list->data;
+		const char  *edit_name;
+		const char  *end_pos;
+
+		g_signal_handlers_block_by_func (GET_WIDGET ("template_entry"), update_preview_cb, data);
+		gtk_entry_set_text (GTK_ENTRY (GET_WIDGET ("template_entry")), g_file_info_get_attribute_string (file_data->info, G_FILE_ATTRIBUTE_STANDARD_EDIT_NAME));
+		g_signal_handlers_unblock_by_func (GET_WIDGET ("template_entry"), update_preview_cb, data);
+
+		gtk_widget_grab_focus (GET_WIDGET ("template_entry"));
+
+		edit_name = gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("template_entry")));
+		end_pos = g_utf8_strrchr (edit_name, -1, '.');
+		if (end_pos != NULL) {
+			glong nchars;
+
+			nchars = g_utf8_strlen (edit_name, (gssize) (end_pos - edit_name));
+			gtk_editable_select_region (GTK_EDITABLE (GET_WIDGET ("template_entry")), 0, nchars);
+		}
+	}
+
+	data->first_update = FALSE;
 
 	if (data->new_names_list != NULL) {
 		_g_string_list_free (data->new_names_list);
@@ -325,7 +398,7 @@ dlg_rename_series_update_preview (DialogData *data)
 		data->new_file_list = NULL;
 	}
 
-	data->new_file_list = g_list_copy (data->file_list);
+	data->new_file_list = g_list_copy (data->file_data_list);
 	if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (data->sort_combobox), &iter)) {
 		GthFileDataSort *sort_type;
 
@@ -376,8 +449,248 @@ dlg_rename_series_update_preview (DialogData *data)
 	g_regex_unref (re);
 	data->new_names_list = g_list_reverse (data->new_names_list);
 
+	if (update_data->ready_func)
+		update_data->ready_func (error, update_data->data);
+
+	update_data_free (update_data);
+}
+
+
+static void
+load_file_data_task_completed_cb (GthTask  *task,
+				  GError   *error,
+				  gpointer  user_data)
+{
+	UpdateData *update_data = user_data;
+	DialogData *data = update_data->data;
+
+	gtk_widget_hide (GET_WIDGET ("task_box"));
+	gtk_widget_set_sensitive (GET_WIDGET ("options_table"), TRUE);
+	gtk_widget_set_sensitive (GET_WIDGET ("ok_button"), TRUE);
+
+	data->tasks = g_list_remove (data->tasks, update_data->task);
+
+	g_object_unref (update_data->task);
+	update_data->task = NULL;
+	update_data->task_completed_id = 0;
+
 	if (error != NULL) {
-		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (data->dialog), _("Could not rename the files"), &error);
+		if (! data->closing && update_data->ready_func)
+			update_data->ready_func (error, update_data->data);
+		update_data_free (update_data);
+
+		if (data->tasks == NULL)
+			destroy_dialog (data);
+
+		return;
+	}
+
+	_g_object_list_unref (data->file_data_list);
+	data->file_data_list = _g_object_list_ref (gth_load_file_data_task_get_result (GTH_LOAD_FILE_DATA_TASK (task)));
+	data->template_changed = FALSE;
+
+	update_file_list__step2 (update_data);
+}
+
+
+static void
+update_file_list (DialogData *data,
+		  ReadyFunc   ready_func)
+{
+	UpdateData *update_data;
+
+	update_data = g_new (UpdateData, 1);
+	update_data->data = data;
+	update_data->ready_func = ready_func;
+
+	if (data->template_changed) {
+		char     *required_attributes;
+		gboolean  reload_required;
+
+		required_attributes = get_required_attributes (data);
+		reload_required = attribute_list_reload_required (data->required_attributes, required_attributes);
+		g_free (data->required_attributes);
+		data->required_attributes = required_attributes;
+
+		if (reload_required) {
+			GtkWidget *child;
+
+			gtk_widget_set_sensitive (GET_WIDGET ("options_table"), FALSE);
+			gtk_widget_set_sensitive (GET_WIDGET ("ok_button"), FALSE);
+			gtk_widget_show (GET_WIDGET ("task_box"));
+
+			update_data->task = gth_load_file_data_task_new (data->file_list, data->required_attributes);
+			update_data->task_completed_id = g_signal_connect (update_data->task,
+									   "completed",
+									   G_CALLBACK (load_file_data_task_completed_cb),
+									   update_data);
+
+			data->tasks = g_list_prepend (data->tasks, update_data->task);
+
+			child = gth_task_progress_new (update_data->task);
+			gtk_widget_show (child);
+			gtk_box_pack_start (GTK_BOX (GET_WIDGET ("task_box")), child, TRUE, TRUE, 0);
+			gth_task_exec (update_data->task, NULL);
+
+			return;
+		}
+	}
+
+	call_when_idle (update_file_list__step2, update_data);
+}
+
+
+static void
+ok_button_clicked__step2 (GError   *error,
+			  gpointer  user_data)
+{
+	DialogData  *data = user_data;
+	GtkTreeIter  iter;
+	GList       *old_files;
+	GList       *new_files;
+	GList       *scan1;
+	GList       *scan2;
+	GthTask     *task;
+
+	if (error != NULL) {
+		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (data->dialog), _("Could not rename the files"), error);
+		return;
+	}
+
+	/* -- save preferences -- */
+
+	if (data->file_list->next != NULL)
+		eel_gconf_set_string (PREF_RENAME_SERIES_TEMPLATE, gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("template_entry"))));
+
+	eel_gconf_set_integer (PREF_RENAME_SERIES_START_AT, gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (GET_WIDGET ("start_at_spinbutton"))));
+
+	if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (data->sort_combobox), &iter)) {
+		GthFileDataSort *sort_type;
+
+		gtk_tree_model_get (GTK_TREE_MODEL (data->sort_model),
+				    &iter,
+				    SORT_DATA_COLUMN, &sort_type,
+				    -1);
+		eel_gconf_set_string (PREF_RENAME_SERIES_SORT_BY, sort_type->name);
+	}
+
+	eel_gconf_set_boolean (PREF_RENAME_SERIES_REVERSE_ORDER, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("reverse_order_checkbutton"))));
+	eel_gconf_set_integer (PREF_RENAME_SERIES_CHANGE_CASE, gtk_combo_box_get_active (GTK_COMBO_BOX (data->change_case_combobox)));
+
+	/* -- prepare and exec rename task -- */
+
+	old_files = NULL;
+	new_files = NULL;
+
+	for (scan1 = data->new_file_list, scan2 = data->new_names_list;
+	     scan1 && scan2;
+	     scan1 = scan1->next, scan2 = scan2->next)
+	{
+		GthFileData *file_data = scan1->data;
+		char        *new_name  = scan2->data;
+		GFile       *parent;
+		GFile       *new_file;
+
+		parent = g_file_get_parent (file_data->file);
+		new_file = g_file_get_child (parent, new_name);
+
+		old_files = g_list_prepend (old_files, g_object_ref (file_data->file));
+		new_files = g_list_prepend (new_files, new_file);
+
+		g_object_unref (parent);
+	}
+	old_files = g_list_reverse (old_files);
+	new_files = g_list_reverse (new_files);
+
+	task = gth_rename_task_new (old_files, new_files);
+	gth_browser_exec_task (data->browser, task, FALSE);
+
+	g_object_unref (task);
+	destroy_dialog (data);
+}
+
+
+static void
+ok_button_clicked (DialogData *data)
+{
+	if (data->update_id != 0) {
+		g_source_remove (data->update_id);
+		data->update_id = 0;
+	}
+
+	update_file_list (data, ok_button_clicked__step2);
+}
+
+
+static void
+dialog_response_cb (GtkDialog *dialog,
+		    int        response_id,
+		    gpointer   user_data)
+{
+	DialogData *data = user_data;
+
+	switch (response_id) {
+	case GTK_RESPONSE_DELETE_EVENT:
+	case GTK_RESPONSE_CANCEL:
+		if (data->tasks != NULL) {
+			GList *tasks;
+
+			data->closing = TRUE;
+
+			tasks = g_list_copy (data->tasks);
+			g_list_foreach (tasks, (GFunc) gth_task_cancel, NULL);
+			g_list_free (tasks);
+		}
+		else
+			destroy_dialog (data);
+		break;
+
+	case GTK_RESPONSE_HELP:
+		show_help_dialog (GTK_WINDOW (dialog), "gthumb-rename-series");
+		break;
+
+	case GTK_RESPONSE_OK:
+		ok_button_clicked (data);
+		break;
+
+	default:
+		break;
+	}
+}
+
+
+static void
+error_dialog_response_cb (GtkDialog *dialog,
+			  int        response,
+			  gpointer   user_data)
+{
+	DialogData *data = user_data;
+
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+	destroy_dialog (data);
+}
+
+
+static void
+update_preview__step2 (GError   *error,
+		       gpointer  user_data)
+{
+	DialogData *data = user_data;
+	GList      *scan1, *scan2;
+
+	if (error != NULL) {
+		GtkWidget *d;
+
+		d = _gtk_message_dialog_new (GTK_WINDOW (data->dialog),
+					     GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+					     GTK_STOCK_DIALOG_ERROR,
+					     _("Could not rename the files"),
+					     error->message,
+					     GTK_STOCK_OK, GTK_RESPONSE_OK,
+					     NULL);
+		g_signal_connect (d, "response", G_CALLBACK (error_dialog_response_cb), data);
+		gtk_window_present (GTK_WINDOW (d));
+
 		return;
 	}
 
@@ -401,6 +714,13 @@ dlg_rename_series_update_preview (DialogData *data)
 }
 
 
+static void
+dlg_rename_series_update_preview (DialogData *data)
+{
+	update_file_list (data, update_preview__step2);
+}
+
+
 static void
 template_entry_icon_press_cb (GtkEntry             *entry,
                               GtkEntryIconPosition  icon_pos,
@@ -418,11 +738,76 @@ template_entry_icon_press_cb (GtkEntry             *entry,
 }
 
 
+static gboolean
+update_preview_after_delay_cb (gpointer user_data)
+{
+	DialogData *data = user_data;
+
+	if (data->update_id != 0) {
+		g_source_remove (data->update_id);
+		data->update_id = 0;
+	}
+
+	dlg_rename_series_update_preview (data);
+
+	return FALSE;
+}
+
+
 static void
 update_preview_cb (GtkWidget  *widget,
 		   DialogData *data)
 {
-	dlg_rename_series_update_preview (data);
+	data->template_changed = TRUE;
+	if (data->update_id != 0)
+		g_source_remove (data->update_id);
+	data->update_id = g_timeout_add (UPDATE_DELAY, update_preview_after_delay_cb, data);
+}
+
+
+static void
+template_editor_dialog_response_cb (GtkDialog *dialog,
+				    int        response_id,
+				    gpointer   user_data)
+{
+	DialogData *data = user_data;
+	char       *template;
+	GError     *error = NULL;
+
+	if (response_id != GTK_RESPONSE_OK) {
+		gtk_widget_destroy (GTK_WIDGET (dialog));
+		return;
+	}
+
+	template = gth_template_editor_dialog_get_template (GTH_TEMPLATE_EDITOR_DIALOG (dialog), &error);
+	if (error != NULL) {
+		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (dialog), _("Could not save the template"), error);
+		g_clear_error (&error);
+		return;
+	}
+
+	gtk_entry_set_text (GTK_ENTRY (GET_WIDGET ("template_entry")), template);
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+
+	g_free (template);
+}
+
+
+static void
+edit_template_button_clicked_cb (GtkWidget  *widget,
+				 DialogData *data)
+{
+	GtkWidget *dialog;
+
+	dialog = gth_template_editor_dialog_new (Rename_Special_Codes, 8, _("Edit Template"), GTK_WINDOW (data->dialog));
+	gth_template_editor_dialog_set_template (GTH_TEMPLATE_EDITOR_DIALOG (dialog),
+						 gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("template_entry"))));
+	g_signal_connect (dialog,
+			  "response",
+			  G_CALLBACK (template_editor_dialog_response_cb),
+			  data);
+	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+	gtk_window_present (GTK_WINDOW (dialog));
 }
 
 
@@ -449,7 +834,10 @@ dlg_rename_series (GthBrowser *browser,
 	data = g_new0 (DialogData, 1);
 	data->browser = browser;
 	data->builder = _gtk_builder_new_from_file ("rename-series.ui", "rename_series");
-	data->file_list = gth_file_data_list_dup (file_list);
+	data->file_list = _g_file_list_dup (file_list);
+	data->first_update = TRUE;
+	data->template_changed = TRUE;
+	data->closing = FALSE;
 
 	/* Get the widgets. */
 
@@ -464,21 +852,28 @@ dlg_rename_series (GthBrowser *browser,
 					       G_TYPE_STRING);
 	data->list_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (data->list_store));
 	g_object_unref (data->list_store);
+	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (data->list_view), TRUE);
 
 	renderer = gtk_cell_renderer_text_new ();
+	g_object_set (renderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
 	column = gtk_tree_view_column_new_with_attributes (_("Old Name"),
 							   renderer,
 							   "text", PREVIEW_OLD_NAME_COLUMN,
 							   NULL);
 	gtk_tree_view_column_set_expand (GTK_TREE_VIEW_COLUMN (column), TRUE);
+	gtk_tree_view_column_set_resizable (GTK_TREE_VIEW_COLUMN (column), TRUE);
+	gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (column), GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (data->list_view), column);
 
 	renderer = gtk_cell_renderer_text_new ();
+	g_object_set (renderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
 	column = gtk_tree_view_column_new_with_attributes (_("New Name"),
 							   renderer,
 							   "text", PREVIEW_NEW_NAME_COLUMN,
 							   NULL);
 	gtk_tree_view_column_set_expand (GTK_TREE_VIEW_COLUMN (column), TRUE);
+	gtk_tree_view_column_set_resizable (GTK_TREE_VIEW_COLUMN (column), TRUE);
+	gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (column), GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (data->list_view), column);
 
 	gtk_widget_show (data->list_view);
@@ -486,11 +881,7 @@ dlg_rename_series (GthBrowser *browser,
 
 	gtk_label_set_mnemonic_widget (GTK_LABEL (GET_WIDGET ("preview_label")), data->list_view);
 
-	if (data->file_list->next == NULL) {
-		GthFileData *file_data = data->file_list->data;
-		gtk_entry_set_text (GTK_ENTRY (GET_WIDGET ("template_entry")), g_file_info_get_attribute_string (file_data->info, G_FILE_ATTRIBUTE_STANDARD_EDIT_NAME));
-	}
-	else
+	if (data->file_list->next != NULL)
 		gtk_entry_set_text (GTK_ENTRY (GET_WIDGET ("template_entry")), eel_gconf_get_string (PREF_RENAME_SERIES_TEMPLATE, DEFAULT_TEMPLATE));
 
 	start_at = eel_gconf_get_integer (PREF_RENAME_SERIES_START_AT, DEFAULT_START_AT);
@@ -538,7 +929,7 @@ dlg_rename_series (GthBrowser *browser,
 		gtk_combo_box_set_active (GTK_COMBO_BOX (data->sort_combobox), 0);
 
 	gtk_widget_show (data->sort_combobox);
-	gtk_container_add (GTK_CONTAINER (GET_WIDGET ("sort_by_box")), data->sort_combobox);
+	gtk_box_pack_start (GTK_BOX (GET_WIDGET ("sort_by_box")), data->sort_combobox, FALSE, FALSE, 0);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (GET_WIDGET ("sort_by_label")), data->sort_combobox);
 
 	/* reverse order */
@@ -557,29 +948,19 @@ dlg_rename_series (GthBrowser *browser,
 								    NULL);
 	gtk_combo_box_set_active (GTK_COMBO_BOX (data->change_case_combobox), change_case);
 	gtk_widget_show (data->change_case_combobox);
-	gtk_container_add (GTK_CONTAINER (GET_WIDGET ("change_case_box")), data->change_case_combobox);
+	gtk_box_pack_start (GTK_BOX (GET_WIDGET ("change_case_box")), data->change_case_combobox, FALSE, FALSE, 0);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (GET_WIDGET ("change_case_label")), data->change_case_combobox);
 
-	dlg_rename_series_update_preview (data);
-
 	/* Set the signals handlers. */
 
-	g_signal_connect (G_OBJECT (data->dialog),
-			  "destroy",
-			  G_CALLBACK (destroy_cb),
+	g_signal_connect (data->dialog,
+			  "delete-event",
+			  G_CALLBACK (gtk_true),
+			  NULL);
+	g_signal_connect (data->dialog,
+			  "response",
+			  G_CALLBACK (dialog_response_cb),
 			  data);
-	g_signal_connect (GET_WIDGET ("ok_button"),
-			  "clicked",
-			  G_CALLBACK (ok_clicked_cb),
-			  data);
-        g_signal_connect (GET_WIDGET ("help_button"),
-                          "clicked",
-                          G_CALLBACK (help_clicked_cb),
-                          data);
-	g_signal_connect_swapped (GET_WIDGET ("cancel_button"),
-				  "clicked",
-				  G_CALLBACK (gtk_widget_destroy),
-				  G_OBJECT (data->dialog));
 	g_signal_connect (GET_WIDGET ("template_entry"),
   			  "icon-press",
   			  G_CALLBACK (template_entry_icon_press_cb),
@@ -604,6 +985,10 @@ dlg_rename_series (GthBrowser *browser,
 			  "toggled",
 			  G_CALLBACK (update_preview_cb),
 			  data);
+        g_signal_connect (GET_WIDGET ("edit_template_button"),
+                          "clicked",
+                          G_CALLBACK (edit_template_button_clicked_cb),
+                          data);
 
 	/* Run dialog. */
 
@@ -611,17 +996,5 @@ dlg_rename_series (GthBrowser *browser,
 	gtk_window_set_modal (GTK_WINDOW (data->dialog), FALSE);
 	gtk_widget_show (data->dialog);
 
-	if (data->file_list->next == NULL) {
-		const char *edit_name;
-		const char *end_pos;
-
-		edit_name = gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("template_entry")));
-		end_pos = g_utf8_strrchr (edit_name, -1, '.');
-		if (end_pos != NULL) {
-			glong nchars;
-
-			nchars = g_utf8_strlen (edit_name, (gssize) (end_pos - edit_name));
-			gtk_editable_select_region (GTK_EDITABLE (GET_WIDGET ("template_entry")), 0, nchars);
-		}
-	}
+	dlg_rename_series_update_preview (data);
 }
