@@ -60,70 +60,74 @@ static guint mount_removed_event_id = 0;
 
 
 static GList *
-gth_file_source_vfs_get_entry_points (GthFileSource *file_source)
+gth_file_source_vfs_add_special_dir (GList         *list, 
+				     GthFileSource *file_source,
+				     GUserDirectory special_dir)
 {
 	const gchar *path;
+
+	path = g_get_user_special_dir (special_dir);
+	if (path != NULL) {
+		GFile *file;
+		file = g_file_new_for_path (path);
+		if (g_file_query_exists (file, NULL)) {
+			GFileInfo *info;
+			info = gth_file_source_get_file_info (file_source, file, GFILE_BASIC_ATTRIBUTES ",access::*");
+			list = g_list_append (list, gth_file_data_new (file, info));
+			g_object_unref (info);
+		}
+
+		g_object_unref (file);
+	}
+
+	return list;
+}
+
+
+static GList *
+gth_file_source_vfs_add_uri (GList         *list,
+                             GthFileSource *file_source,
+                             const char    *uri,
+			     char          *display_name)
+{
+        GFile       *file;
+        GFileInfo   *info;
+
+        file = g_file_new_for_uri (uri);
+        info = gth_file_source_get_file_info (file_source, file, GFILE_BASIC_ATTRIBUTES ",access::*");
+        g_file_info_set_display_name (info, display_name);
+        list = g_list_append (list, gth_file_data_new (file, info));
+        g_object_unref (info);
+        g_object_unref (file);
+
+        return list;
+}
+
+
+static GList *
+gth_file_source_vfs_get_entry_points (GthFileSource *file_source)
+{
 	GList       *list;
-	GFile       *file;
-	GFileInfo   *info;
 	GList       *mounts;
 	GList       *scan;
 
 	list = NULL;
 
-	file = g_file_new_for_uri (get_home_uri ());
-	info = gth_file_source_get_file_info (file_source, file, GFILE_BASIC_ATTRIBUTES ",access::*");
-	g_file_info_set_display_name (info, _("Home Folder"));
-	list = g_list_append (list, gth_file_data_new (file, info));
-	g_object_unref (info);
-	g_object_unref (file);
-
-	path = g_get_user_special_dir (G_USER_DIRECTORY_PICTURES);
-	if (path != NULL) {
-		file = g_file_new_for_path (path);
-		if (g_file_query_exists (file, NULL)) {
-			info = gth_file_source_get_file_info (file_source, file, GFILE_BASIC_ATTRIBUTES ",access::*");
-			list = g_list_append (list, gth_file_data_new (file, info));
-			g_object_unref (info);
-		}
-		g_object_unref (file);
-	}
-
-	path = g_get_user_special_dir (G_USER_DIRECTORY_VIDEOS);
-	if (path != NULL) {
-		file = g_file_new_for_path (path);
-		if (g_file_query_exists (file, NULL)) {
-			info = gth_file_source_get_file_info (file_source, file, GFILE_BASIC_ATTRIBUTES ",access::*");
-			list = g_list_append (list, gth_file_data_new (file, info));
-			g_object_unref (info);
-		}
-		g_object_unref (file);
-	}
-
-	path = g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP);
-	if (path != NULL) {
-		file = g_file_new_for_path (path);
-		if (g_file_query_exists (file, NULL)) {
-			info = gth_file_source_get_file_info (file_source, file, GFILE_BASIC_ATTRIBUTES ",access::*");
-			list = g_list_append (list, gth_file_data_new (file, info));
-			g_object_unref (info);
-		}
-		g_object_unref (file);
-	}
-
-	file = g_file_new_for_uri ("file:///");
-	info = gth_file_source_get_file_info (file_source, file, GFILE_BASIC_ATTRIBUTES ",access::*");
-	g_file_info_set_display_name (info, _("File System"));
-	list = g_list_append (list, gth_file_data_new (file, info));
-	g_object_unref (info);
-	g_object_unref (file);
+	list = gth_file_source_vfs_add_uri (list, file_source, get_home_uri (), _("Home Folder"));
+	list = gth_file_source_vfs_add_special_dir (list, file_source, G_USER_DIRECTORY_PICTURES);
+	list = gth_file_source_vfs_add_special_dir (list, file_source, G_USER_DIRECTORY_VIDEOS);
+	list = gth_file_source_vfs_add_special_dir (list, file_source, G_USER_DIRECTORY_DESKTOP);
+	list = gth_file_source_vfs_add_special_dir (list, file_source, G_USER_DIRECTORY_DOCUMENTS);
+        list = gth_file_source_vfs_add_uri (list, file_source, "file:///", _("File System"));
 
 	mounts = g_volume_monitor_get_mounts (g_volume_monitor_get ());
 	for (scan = mounts; scan; scan = scan->next) {
-		GMount  *mount = scan->data;
-		GIcon   *icon;
-		char    *name;
-		GDrive  *drive;
+		GMount    *mount = scan->data;
+		GIcon     *icon;
+		char      *name;
+		GDrive    *drive;
+		GFile     *file;
+		GFileInfo *info;
 
 		if (g_mount_is_shadowed (mount))
 			continue;
@@ -338,6 +342,7 @@ gth_file_source_vfs_copy (GthFileSource    *file_source,
 			  GthFileData      *destination,
 			  GList            *file_list, /* GFile * list */
 			  gboolean          move,
+			  int               destination_position,
 			  ProgressCallback  progress_callback,
 			  DialogCallback    dialog_callback,
 		          ReadyCallback     ready_callback,
@@ -615,19 +620,56 @@ gth_file_source_vfs_monitor_directory (GthFileSource *file_source,
 /* -- gth_file_source_vfs_remove -- */
 
 
+static void
+parent_table_value_free (gpointer key,
+			 gpointer value,
+			 gpointer user_data)
+{
+	_g_object_list_unref (value);
+}
+
+
 static void
 notify_files_delete (GtkWindow *window,
 		     GList     *files)
 {
-	GFile *parent;
+	GHashTable *parent_table;
+	GList      *scan;
+	GList      *parent_list;
 
-	parent = g_file_get_parent ((GFile*) files->data);
-	gth_monitor_folder_changed (gth_main_get_default_monitor (),
-				    parent,
-				    files,
-				    GTH_MONITOR_EVENT_DELETED);
+	parent_table = g_hash_table_new_full (g_file_hash, (GEqualFunc) g_file_equal, g_object_unref, NULL);
+	for (scan = files; scan; scan = scan->next) {
+		GFile *file = scan->data;
+		GFile *parent;
+		GList *children;
 
-	g_object_unref (parent);
+		parent = g_file_get_parent (file);
+		if (parent == NULL)
+			return;
+
+		children = g_hash_table_lookup (parent_table, parent);
+		children = g_list_prepend (children, g_object_ref (file));
+		g_hash_table_replace (parent_table, g_object_ref (parent), children);
+
+		g_object_unref (parent);
+	}
+
+	parent_list = g_hash_table_get_keys (parent_table);
+	for (scan = parent_list; scan; scan = scan->next) {
+		GFile *parent = scan->data;
+		GList *children;
+
+		children = g_hash_table_lookup (parent_table, parent);
+		if (children != NULL)
+			gth_monitor_folder_changed (gth_main_get_default_monitor (),
+						    parent,
+						    children,
+						    GTH_MONITOR_EVENT_DELETED);
+	}
+
+	g_list_free (parent_list);
+	g_hash_table_foreach (parent_table, parent_table_value_free, NULL);
+	g_hash_table_unref (parent_table);
 }
 
 
@@ -639,8 +681,10 @@ delete_file_permanently (GtkWindow *window,
 	GError *error = NULL;
 
 	files = gth_file_data_list_to_file_list (file_list);
-	if (! _g_delete_files (files, TRUE, &error))
-		_gtk_error_dialog_from_gerror_show (window, _("Could not delete the files"), &error);
+	if (! _g_delete_files (files, TRUE, &error)) {
+		_gtk_error_dialog_from_gerror_show (window, _("Could not delete the files"), error);
+		g_clear_error (&error);
+	}
 	else
 		notify_files_delete (window, files);
 
@@ -694,7 +738,8 @@ trash_files (GtkWindow *window,
 
 				break;
 			}
-			_gtk_error_dialog_from_gerror_show (window, _("Could not move the files to the Trash"), &error);
+			_gtk_error_dialog_from_gerror_show (window, _("Could not move the files to the Trash"), error);
+			g_clear_error (&error);
 			break;
 		}
 	}
