@@ -148,7 +148,6 @@ get_tag_limits (GthTagsEntry  *self,
 	const char *cursor_start;
 	const char *tag_start;
 	const char *tag_end;
-	const char *tmp_tag_end;
 
 	text = gtk_entry_get_text (GTK_ENTRY (self->priv->entry));
 	cursor_start = g_utf8_offset_to_pointer (text, gtk_editable_get_position (GTK_EDITABLE (self->priv->entry)));
@@ -174,10 +173,6 @@ get_tag_limits (GthTagsEntry  *self,
 	while ((tag_start != tag_end) && g_unichar_isspace (g_utf8_get_char (tag_start)))
 		tag_start = g_utf8_next_char (tag_start);
 
-	tmp_tag_end = g_utf8_strrchr (tag_start, tag_end - tag_start, ' ');
-	if (tmp_tag_end != NULL)
-		tag_end = tmp_tag_end;
-
 	*tag_start_p = tag_start;
 	*tag_end_p = tag_end;
 }
@@ -849,7 +844,7 @@ gth_tags_entry_get_tags (GthTagsEntry *self,
 	for (i = 0, j = 0; all_tags[i] != NULL; i++) {
 		all_tags[i] = g_strstrip (all_tags[i]);
 		if (all_tags[i][0] != '\0') {
-			tags[j] = g_strdup (all_tags[i]);
+			tags[j] = g_strdup (g_strstrip (all_tags[i]));
 			if (update_globals)
 				gth_tags_file_add (tags_file, tags[j]);
 			j++;
@@ -871,14 +866,24 @@ void
 gth_tags_entry_set_tags (GthTagsEntry  *self,
 			 char         **tags)
 {
-	char *s;
+	GthTagsFile *tags_file;
+	int          i;
+	gboolean     global_tags_changed = FALSE;
+	char        *s;
 
 	if ((tags == NULL) || (tags[0] == NULL)) {
 		gtk_entry_set_text (GTK_ENTRY (self->priv->entry), "");
 		return;
 	}
 
-	s = g_strjoinv(", ", tags);
+	tags_file = gth_main_get_default_tag_file ();
+	for (i = 0; tags[i] != NULL; i++)
+		if (gth_tags_file_add (tags_file, tags[i]))
+			global_tags_changed = TRUE;
+	if (global_tags_changed)
+		gth_main_tags_changed ();
+
+	s = g_strjoinv (", ", tags);
 	gtk_entry_set_text (GTK_ENTRY (self->priv->entry), s);
 	g_free (s);
 }
@@ -888,5 +893,15 @@ void
 gth_tags_entry_set_text (GthTagsEntry *self,
 			 const char   *text)
 {
-	gtk_entry_set_text (GTK_ENTRY (self->priv->entry), text);
+	char **tags;
+
+	if ((text == NULL) || (strcmp (text, "") == 0)) {
+		gth_tags_entry_set_tags (self, NULL);
+		return;
+	}
+
+	tags = g_strsplit (text, ",", -1);
+	gth_tags_entry_set_tags (self, tags);
+
+	g_strfreev (tags);
 }
